version = "1.1";

-- Machine settings
screen.orientation(0); -- set to landscape view

-- Global display settings
color_bg    = color.new(0,0,0);
color_red   = color.new(255,0,0);
color_touch = color.new(255,100,0);
color_snake = color.new(255,255,255);
color_candy = color.new(0,255,0);
color_grey  = color.new(128,128,128);

--unit_size = 10; -- size of unit
--block_size = 8; -- size of block that goes into the unit
--game_wait = 10;
--add_length = 3;
--candies = 1;


-- Global program variables
lang = 0;
menu_lang=image.load("img/menu_lang.png")
rumenu_main=image.load("img/menu_ru.png")
rumenu_easy=image.load("img/menu_easy_ru.png")
rumenu_sred=image.load("img/menu_sred_ru.png")
rumenu_hard=image.load("img/menu_hard_ru.png")
rumenu_record=image.load("img/menu_record_ru.png")
rumenu_exit=image.load("img/menu_exit_ru.png")
credits_ru=image.load("img/credits_ru.png")

engmenu_main=image.load("img/menu_eng.png")
engmenu_easy=image.load("img/menu_easy_eng.png")
engmenu_sred=image.load("img/menu_sred_eng.png")
engmenu_hard=image.load("img/menu_hard_eng.png")
engmenu_record=image.load("img/menu_record_eng.png")
engmenu_exit=image.load("img/menu_exit_eng.png")
credits_eng=image.load("img/credits_eng.png")

sound_gameover = wav.load("music/gameover.wav");
sound_eat = wav.load("music/eat.wav");
sound_turn = wav.load("music/turn.wav");
sound_click = wav.load("music/click.wav");
menu = true;
playing = false;
continue = true;
touch_pos = {0,0}; -- the location the screen is touched
highscores = {0,0,0,0,0,0,0,0,0,0};
highscores["length"] = 10;

-- Function to (re)set snake related game variables
function snakeVariables()
  block_margin = math.floor((unit_size-block_size)/2);
  units = {x=math.floor(400/unit_size), y=math.floor(240/unit_size)};
  -- Create empty field
  field = {}; -- field with a 0 on empty and negative number on deadly locations
  for i = 1,units.x do -- for blocks on x axis (units.x blocks of unit_size pixels)
    field[i] = {};
    for j = 1,units.y do -- for blocks on y axis (units.y blocks of unit_size pixels)
      field[i][j] = 0;
    end;
  end;
  snake_head = {x=1, y=math.floor(units.y/2)}; -- location of snakes head
  snake_length = add_length; -- length of snake
  snake_dir = {x=1, y=0}; -- movement snake in x,y direction
  snake_pos = {key=0}; -- position of snake parts, used to remove end of snake
  -- Initialize snake
  for i = 1,snake_length do -- draw small snake to start with
    snake_pos.key = i; -- do not use key as counter (i), counter is local variable
    snake_head.x = snake_head.x + 1;
    snake_pos[snake_pos.key] = {snake_head.x, snake_head.y};
    field[snake_head.x][snake_head.y] = -1;
    newSnakePart();
  end;
  snake_pos.key = 1; -- position key for tail position in snake_pos list
end;



-- Save scores to file if the score is higher
function saveScores()
  if highscores[highscores["length"]] < snake_length then
    highscores[highscores["length"]] = snake_length; -- number of highscores saved could be more than 10
    table.sort(highscores, function(a,b) return a>b end);
    local file = io.open("highscores.lua", "w");
    if file ~= nil then
      file:write("# Snake Highscores:\n");
      for key,value in ipairs(highscores) do
        file:write(value,"\n")
      end;
    end;
    file:close();
  end;
end;

-- Load scores from file
function loadScores()
  local file = io.open("highscores.lua", "r");
  if file ~= nil then
    input = file:read("*l");
    if (input ~= nil) or (string.find(input, "Snake") ~= nil) then
      input = file:read("*l");
      local i = 0;
      while input ~= nil do
        i = i + 1;
        highscores[i] = tonumber(input);
        input = file:read("*l");
      end;
      highscores["length"] = i;
    end;
  end;
end;

-- Draw the background to frame buffer
function clearScreen()
  screen.fillrect(0, 0, 400, 240, color_bg); -- a.t.m. the background is just one color
end;

-- Draw block to frame buffer
function drawBlock(x, y, color, hollow)
  local x = (x-1)*unit_size+block_margin;
  local y = (y-1)*unit_size+block_margin;
  screen.fillrect(x, y, block_size, block_size, color);
  -- rounded edges
  if true then
    screen.drawpixel(x, y, color_bg);
    screen.drawpixel(x+block_size-1, y, color_bg);
    screen.drawpixel(x, y+block_size-1, color_bg);
    screen.drawpixel(x+block_size-1, y+block_size-1, color_bg);
  end;
  -- draw rectangle in background color in the middle of the block
  if hollow == true then
    screen.fillrect(x+block_size/4, y+block_size/4, block_size/2, block_size/2, color_bg);
  end;
  --text.size(block_size); text.color(color); text.draw(x, y, "@"); -- alternative method
end;

-- Draw snake part to frame buffer
function newSnakePart()
  drawBlock(snake_head.x, snake_head.y, color_snake, input);
end;

-- Delete snake part from frame buffer (draw background over snake part)
function delSnakePart()
  screen.fillrect((snake_pos[snake_pos.key][1]-1)*unit_size, (snake_pos[snake_pos.key][2]-1)*unit_size, unit_size, unit_size, color_bg);
end;

-- Draw new food for snake to frame buffer
function newCandy()
  local x, y; -- candy location
  repeat
    x = math.random(1,units.x);
    y = math.random(1,units.y);
  until field[x][y] == 0;
  drawBlock(x, y, color_candy, true); -- draw to buffer
  field[x][y] = add_length;
end;

-- Draw background/field to frame buffer
function drawField()
  for x = 1,units.x do
    for y = 1,units.y do
      if field[x][y] == -2 then
        local X = (x-1)*unit_size;
        local Y = (y-1)*unit_size;
        screen.fillrect(X, Y, unit_size, unit_size, color_grey);
  --      screen.fillrect(X+1, Y+1, unit_size/2-1, unit_size/2-1, color_red);
  --      screen.fillrect(X+unit_size/2+1, Y+unit_size/2+1, unit_size/2-2, unit_size/2-2, color_red);
      end;
    end;
  end;
end;

-- Check if touch changes direction
function changeDirection() -- bounds depend on snake position
  if snake_dir.x == 0 then
    if touch_pos[1] < (100+unit_size/2+(snake_head.x-1)*unit_size/2) then
	  sound_turn:play();
      snake_dir.x, snake_dir.y = -1, 0; --up
    else
	  sound_turn:play();
      snake_dir.x, snake_dir.y = 1, 0; --down
    end;
  else
    if touch_pos[2] < (60+unit_size/2+(snake_head.y-1)*unit_size/2) then
	  sound_turn:play();
      snake_dir.x, snake_dir.y = 0, -1; --left
    else
	  sound_turn:play();
      snake_dir.x, snake_dir.y = 0, 1; --right
    end;
  end;
end;

-- Display score
function endMessage(message)
  text.color(color_candy);
  text.size(40);
  text.draw(0,30,message,"center");
  text.size(80);
  text.draw(0,75,tostring(snake_length),"center");
end;

-- Game over message
function gameOver()
  sound_gameover:play();
  local message = "";
  if lang == 2 then
	if snake_length > highscores[1] then
		message = " !";
	elseif snake_length > highscores[10] then
		message = " 10 ";
	end;
  elseif lang == 1 then
    if snake_length > highscores[1] then
		message = "New Highscore!";
	elseif snake_length > highscores[10] then
		message = "Top 10 score";
	end;
  end;
	
  for i = 1,120 do
    drawBlock(math.random(1,units.x), math.random(1,units.y), color_red, false);
    drawBlock(math.random(1,units.x), math.random(1,units.y), color_red, false);
    drawBlock(math.random(1,units.x), math.random(1,units.y), color_red, false);
    drawBlock(math.random(1,units.x), math.random(1,units.y), color_red, false);
    endMessage(message);
    screen.update();
    os.sleep(1);
  end;
  os.sleep(50);
end;

-- Check if location is touched
function touchCheck(x, y, w, h)
  if touch_pos[1]>=x and touch_pos[1]<=(x+w) and touch_pos[2]>=y and touch_pos[2]<=(y+h) then
    return true;
  else
    return false;
  end;
end;

-- Draw main menu to screen
function menuMain()
  if lang == 0 then
	menu_lang:draw(0,0);
	screen.update();
	os.sleep(5);
	if touchCheck(37, 92, 150, 90) then
		sound_click:play();
		lang = 1;
		touch_pos = {0,0};
		control.read();
	end;
	if touchCheck(217, 92, 150, 90) then
		sound_click:play();
		lang = 2;
		touch_pos = {0,0};
		control.read();
	end;
  elseif lang == 2 then
	-- Title
		rumenu_main:draw(0,0);
		screen.update();
		os.sleep(4);
		-- Start game button 1
		if touchCheck(30, 75, 100, 38) then
			sound_click:play();
			rumenu_easy:draw(0,0);
			screen.update();
			os.sleep(10);
			menu, playing = false, true;
			unit_size = 20; block_size = 16; game_wait = 20; add_length = 1; candies = 1;
		end;
		if touchCheck(150, 75, 100, 38) then
			sound_click:play();
			rumenu_sred:draw(0,0);
			screen.update();
			os.sleep(10);
			menu, playing = false, true;
			unit_size = 10; block_size = 8;  game_wait = 10; add_length = 3; candies = 2;
			color = color_touch;
		end;
		if touchCheck(270, 75, 100, 38) then
			sound_click:play();
			rumenu_hard:draw(0,0);
			screen.update();
			os.sleep(10);
			menu, playing = false, true;
			unit_size = 5;  block_size = 4;  game_wait = 5; add_length = 5; candies = 5;
			color = color_touch;
		end;
		-- Highscores button
		color = color_red;
		if touchCheck(100, 130, 200, 38) then
			sound_click:play();
			rumenu_record:draw(0,0);
			screen.update();
			os.sleep(10);
			menu = 2;
			color = color_touch;
		end;
  -- Quit game button
		if touchCheck(100, 175, 200, 38) then
			sound_click:play();
			rumenu_exit:draw(0,0);
			screen.update();
			os.sleep(10);
			credits_ru:draw(0,0);
			screen.update();
			os.sleep(300);
			menu, playing, continue = false;
			color = color_touch;
		end;
		screen.update();
		os.sleep(10);
	elseif lang == 1 then
	
	-- Title
		engmenu_main:draw(0,0);
		screen.update();
		os.sleep(4);
		-- Start game button 1
		if touchCheck(30, 75, 100, 38) then
			sound_click:play();
			engmenu_easy:draw(0,0);
			screen.update();
			os.sleep(10);
			menu, playing = false, true;
			unit_size = 20; block_size = 16; game_wait = 20; add_length = 1; candies = 1;
		end;
		if touchCheck(150, 75, 100, 38) then
			sound_click:play();
			engmenu_sred:draw(0,0);
			screen.update();
			os.sleep(10);
			menu, playing = false, true;
			unit_size = 10; block_size = 8;  game_wait = 10; add_length = 3; candies = 2;
			color = color_touch;
		end;
		if touchCheck(270, 75, 100, 38) then
			sound_click:play();
			engmenu_hard:draw(0,0);
			screen.update();
			os.sleep(10);
			menu, playing = false, true;
			unit_size = 5;  block_size = 4;  game_wait = 5; add_length = 5; candies = 5;
			color = color_touch;
		end;
		-- Highscores button
		color = color_red;
		if touchCheck(100, 130, 200, 38) then
			sound_click:play();
			engmenu_record:draw(0,0);
			screen.update();
			os.sleep(10);
			menu = 2;
			color = color_touch;
		end;
  -- Quit game button
		if touchCheck(100, 175, 200, 38) then
			sound_click:play();
			engmenu_exit:draw(0,0);
			screen.update();
			os.sleep(10);
			credits_eng:draw(0,0);
			screen.update();
			os.sleep(300);
			menu, playing, continue = false;
			color = color_touch;
		end;
		screen.update();
		os.sleep(10);
	end;
end;

-- Draw highscores to screen
function menuHighscores()
  text.size(16);
  text.color(color_candy);
  if lang == 2 then
	text.draw(20, 20, ":");
  elseif lang == 1 then
    text.draw(20, 20, "Highscores:");
  end;
  for i = 1,10 do
    text.draw(110, 20*i, tostring(i)..": ");
    text.draw(138, 20*i, tostring(highscores[i]));
  end;
  if touchCheck(20, 20, 360, 200) then
    menu = 1;
  end;
  screen.update();
end;

-- check for input
function inputCheck()
  -- when the screen is touched
  if control.isTouch() == 1 and touch.click() == 1 then
    touch_pos[1],touch_pos[2] = touch.pos();
    if playing == true then
      changeDirection(); -- check for direction change
    elseif menu == true or menu == 1 then
      menuMain(); -- check + draw menu
    elseif menu == 2 then
      menuHighscores();
    end;
    -- clear touch_pos and delete touch.up() event
    touch_pos = {0,0};
    control.read();
  end;
  -- when a button is pressed
  if control.isButton() == 1 then
    sound_click:play();
    -- when the home button is pressed
    if button.home() == 1 and button.click() == 1 then
      if playing == true then
        local message = "";
		  if lang == 2 then
			if snake_length > highscores[1] then
				message = " !";
			elseif snake_length > highscores[10] then
				message = " 10 ";
			end;
		  elseif lang == 1 then
			if snake_length > highscores[1] then
				message = "New Highscore!";
			elseif snake_length > highscores[10] then
				message = "Top 10 score";
			end;
		 end;
        saveScores();
        endMessage(message);
        screen.update();
        os.sleep(40);
        menu, playing = true, false; -- go to menu
      else
	    if lang == 2 then
			credits_ru:draw(0,0);
			screen.update();
			os.sleep(300);
		elseif lang == 1 then
			credits_eng:draw(0,0);
			screen.update();
			os.sleep(300);
		end;
        menu, playing, continue = false; -- exit all
      end;
    -- when the power button is pressed
    elseif button.power() == 1 and playing == true then -- something to pause game...
      control.read(1);
    end;
  end;
end;

-- Background for game (levels)
function gameBackground()
--  file = assert(io.open("snakemenu.txt", "r"));
--  y = 1;
--  if (file ~= nil) then
--    for input in file:lines() do
--      for x = 1,string.len(input) do
--        field[x][y] = tonumber(string.sub(input, x, x));
--      end;
--      y = y + 1;
--    end;
--  end;
--  file:close();

  --for i = 1,units.x do field[i][1] = -2; field[i][units.y] = -2; end; -- upper and lower wall
  --for i = 1,units.y do field[1][i] = -2; field[units.x][i] = -2; end; -- left and right wall
  --for i = 10,units.x-10 do field[i][units.y/2] = -2; end; -- horizontal wall
  --for i = 10,units.y-10 do field[units.x/2][i] = -2; end; -- vertical wall
  --field[units.x/2][1] = 0; field[units.x/2-1][2] = -2; field[units.x/2+1][2] = -2;
  --field[units.x/2][units.y] = 0; field[units.x/2-1][units.y-1] = -2; field[units.x/2+1][units.y-1] = -2;

  field[units.x/2-2][units.y/2-1] = -2; field[units.x/2-2][units.y/2-2] = -2; field[units.x/2-1][units.y/2-2] = -2;
  field[units.x/2-2][units.y/2+1] = -2; field[units.x/2-2][units.y/2+2] = -2; field[units.x/2-1][units.y/2+2] = -2;
  field[units.x/2+2][units.y/2+1] = -2; field[units.x/2+2][units.y/2+2] = -2; field[units.x/2+1][units.y/2+2] = -2;
  field[units.x/2+2][units.y/2-1] = -2; field[units.x/2+2][units.y/2-2] = -2; field[units.x/2+1][units.y/2-2] = -2;
end;



--
-- NOW WE ARE READY TO START THE PROGRAM
-- First the highscores are loaded
-- Followed by a program loop with a menu and game loop
--
loadScores();

-- Program loop
while continue == true do

  -- Empty input queue (or better: delete first 10 items from input queue)
  for i = 1,10 do
    control.read();
  end;

  -- Menu loop
  while menu ~= false do
    clearScreen();
    if menu == 2 then -- draw highscores to screen
      menuHighscores();
    else -- draw main menu to screen
      menuMain();
    end;
    -- wait for input
    if control.read(1) == 1 then
      inputCheck();
    end;
  end;

  -- Prepare to start play loop
  if playing == true then
    clearScreen();
    snakeVariables(); -- (Re)set snake
    gameBackground(); drawField(); -- Get level background & draw background
    for i = 1,candies do
      newCandy(); -- Something to collect
    end;
    screen.update();
    os.sleep(50);
  end;

  -- Game loop
  while playing == true do
    screen.update(); -- show changes made in buffer on screen
    os.sleep(game_wait); -- wait for a given time

    -- check for input
    if control.read() == 1 then
      inputCheck();
    end;

    -- new location for head of snake
    snake_head.x = snake_head.x + snake_dir.x;
    snake_head.y = snake_head.y + snake_dir.y;

    -- no walls
    if snake_head.x < 1 then
      snake_head.x = units.x;
    elseif snake_head.x > units.x then
      snake_head.x = 1;
    elseif snake_head.y < 1 then
      snake_head.y = units.y;
    elseif snake_head.y > units.y then
      snake_head.y = 1;
    end; -- else the snake didn't leave the screen

    -- check if new location is not empty (1 in field matrix)
    if field[snake_head.x][snake_head.y] < 0 then
      gameOver();
      saveScores();
      menu, playing = true, false;
    end;

    -- delete snake tail
    if snake_pos[snake_pos.key][1]~=0 then -- if 0 the snake should grow
      field[ snake_pos[snake_pos.key][1] ][ snake_pos[snake_pos.key][2] ] = 0; -- delete from field
      delSnakePart(); -- delete from screen
    end;

    -- walk over food?
    if field[snake_head.x][snake_head.y] > 0 then
	  sound_eat:play();
      newCandy();
      for i = 1,field[snake_head.x][snake_head.y] do -- increase snake length
        snake_length = snake_length + 1;
        snake_pos[snake_length] = {0,0};
      end;
    end;

    -- add new snake part
    snake_pos[snake_pos.key] = {snake_head.x, snake_head.y};
    field[snake_head.x][snake_head.y] = -1;
    newSnakePart(); -- add to screen
  
    -- add 1 to snake_pos key or make 1 if it exceeds length
    if snake_pos.key < snake_length then
      snake_pos.key = snake_pos.key + 1;
    else
      snake_pos.key = 1;
    end;

  end;

end;
